/*
 * IRCEvent+EventCreation.m
 * Generation of standard protocol events.
 *
 * Copyright (c) 1998 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1998.
 */

#import "IRC.h"

@implementation IRCEvent(EventCreation)

// This should be done in some localized fashion

+ (IRCEvent *)registrationEventWithUser:(NSString *)user userDescription:(NSString *)info
{
    NSArray *arguments = [NSArray arrayWithObjects:user, @"*", @"*", info, nil];
    
    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"USER"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)nicknameChangeEventWithNickname:(NSString *)name
{
    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"NICK"
                                   arguments:[NSArray arrayWithObjects:name, nil]] autorelease];
}

+ (IRCEvent *)joinEventWithChannel:(NSString *)channel key:(NSString *)key
{
    NSArray *arguments = [NSArray arrayWithObjects:channel, key ? key : @"", nil];
    
    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"JOIN"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)messageEventWithTarget:(NSString *)target message:(NSString *)message
{
    NSArray *arguments = [NSArray arrayWithObjects:target, [message stringByAppendingString:@" "], nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"PRIVMSG"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)partEventWithTarget:(NSString *)target
{
    NSArray *arguments = [NSArray arrayWithObjects:target, nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"PART"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)ctcpEventWithTarget:(NSString *)target command:(NSString *)command message:(NSString *)message
{
    NSArray *arguments = [NSArray arrayWithObjects:target,
        [NSString stringWithFormat:@"\001%@ %@\001", command, message],
        nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"PRIVMSG"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)ctcpReplyEventWithTarget:(NSString *)target command:(NSString *)ctcp message:(NSString *)message
{
    NSArray *arguments = [NSArray arrayWithObjects:target,
        [NSString stringWithFormat:@"\001%@ %@\001", ctcp, message],
        nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"NOTICE"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)namesEventWithChannel:(NSString *)channel
{
    NSArray *arguments = [NSArray arrayWithObjects:channel, nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"NAMES"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)pongEventWithServer:(NSString *)server
{
    NSArray *arguments = [NSArray arrayWithObjects:server, nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"PONG"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)whoisEventWithName:(NSString *)name
{
    NSArray *arguments = [NSArray arrayWithObjects:name, nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"WHOIS"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)modeEventWithTarget:(NSString *)target modes:(NSArray *)modes arguments:(NSArray *)args
{
    NSArray *arguments = [NSArray arrayWithObjects:target,
        [modes componentsJoinedByString:@""],
        [args componentsJoinedByString:@""],
        nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"MODE"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)kickEventWithChannel:(NSString *)channel nickname:(NSString *)nickname reason:(NSString *)reason
{
    NSArray *arguments = [NSArray arrayWithObjects:channel,
        nickname,
        [reason stringByPrependingString:@":"],
        nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"KICK"
                                   arguments:arguments] autorelease];
}

+ (IRCEvent *)topicEventWithChannel:(NSString *)channel topic:(NSString *)topic
{
    NSArray *arguments = [NSArray arrayWithObjects:channel, topic, nil];

    return [[[IRCEvent alloc] initWithPrefix:nil
                                     command:@"TOPIC"
                                   arguments:arguments] autorelease];
}

@end
