/*
 * IRCEvent.h
 * Model communications to an IRC server.
 *
 * Copyright (c) 1998 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1998.
 */

@class NSString, NSArray, IRCModel;

IRC_EXPORT NSString *IRCCTCPDelimiterCharacter;

@interface IRCEvent : NSObject <NSCopying, NSCoding>
{
    NSString	*_prefix;
    NSString	*_command;
    NSArray	*_arguments;
}

/* convenience method which will parse the string as best it can */
+ eventWithMessage:(NSString *)aString;

/* designated initializer */
- (id)initWithMessage:(NSString *)message;
- (id)initWithPrefix:(NSString *)prefix command:(NSString *)command arguments:(NSArray *)arguments;

- (NSString *)prefix;
- (NSString *)command;
- (NSArray *)arguments;

/* generates an serverDidCOMMAND:event: selector. 
  first param is the server, second is the event */
- (SEL)selector;

/* data object containing the event's particulars, ready for server transmission */
- (NSString *)string;
- (NSData *)encodedData;

@end

@interface IRCEvent(EventCreation)

+ (IRCEvent *)registrationEventWithUser:(NSString *)user userDescription:(NSString *)info;
+ (IRCEvent *)nicknameChangeEventWithNickname:(NSString *)name;
+ (IRCEvent *)joinEventWithChannel:(NSString *)channel key:(NSString *)key;
+ (IRCEvent *)messageEventWithTarget:(NSString *)target message:(NSString *)message;
+ (IRCEvent *)partEventWithTarget:(NSString *)target;
+ (IRCEvent *)ctcpEventWithTarget:(NSString *)target command:(NSString *)command message:(NSString *)message;
+ (IRCEvent *)ctcpReplyEventWithTarget:(NSString *)target command:(NSString *)ctcp message:(NSString *)message;
+ (IRCEvent *)namesEventWithChannel:(NSString *)channel;
+ (IRCEvent *)whoisEventWithName:(NSString *)name;
+ (IRCEvent *)pongEventWithServer:(NSString *)server;
+ (IRCEvent *)modeEventWithTarget:(NSString *)target modes:(NSArray *)modes arguments:(NSArray *)args;
+ (IRCEvent *)kickEventWithChannel:(NSString *)channel nickname:(NSString *)nickname reason:(NSString *)reason;
+ (IRCEvent *)topicEventWithChannel:(NSString *)channel topic:(NSString *)topic;

@end
