/*
 * IRCEventProcessor.h
 * Event queue manager for an IRCServer.
 *
 * Copyright (c) 1998 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1998.
 */

@class IRCServer, TCStreamMonitor;

@interface IRCEventProcessor : NSObject
{
    IRCServer *_server;
    TCSocket *_socket;
    NSPort *_dataPort;

    NSException *_threadException;
    NSMutableArray *_responders;
}

- (id)initWithServer:(IRCServer *)server;

- (void)addResponder:(id)responder;
- (void)removeResponder:(id)responder;

- (NSArray *)responders;

- (void)dispatchEvent:(IRCEvent *)event responder:(id)responder;
- (void)dispatchEvent:(IRCEvent *)event;

@end

// A responder can override the default ircSELECTOR dispatch mechanism by implementing
// this method. If the method returns YES, the standard dispatch mechanism will run
// anyway.
@interface NSObject(IRCResponder)
- (BOOL)server:(IRCServer *)server willProcessEvent:(IRCEvent *)event;
@end
