/*
 * IRCServer.h
 * Object model of an IRC server.
 *
 * Copyright (c) 1998 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1998.
 */

@class NSString, NSMutableDictionary;
@class IRCChannel, IRCEventProcessor, IRCUser;

/* This notification is actually inherited from the IRCModel superclass */
IRC_EXPORT NSString *IRCServerDidChangeNotification;
IRC_EXPORT NSString *IRCServerChannelListDidChangeNotification;
IRC_EXPORT NSString *IRCServerUserListDidChangeNotification;

IRC_EXPORT NSString *IRCServerDidConnectNotification;
IRC_EXPORT NSString *IRCServerWillTerminateNotification;

@interface IRCServer : IRCModel
{
    NSString *_description;
    NSString *_serverHost;
    int _serverPort;

    IRCUser *_localUser;
    
    NSMutableDictionary	*_channelDictionary;
    NSMutableArray	*_users;

    /* These objects are volatile and not coded */
    TCSocket *_socket;
    IRCEventProcessor *_eventProcessor;
}

/* Conveniences */
+ (IRCServer *)serverWithHostName:(NSString *)hostName port:(int)port nickname:(NSString *)nickname userDescription:(NSString *)userDescription;

- (id)initWithHostName:(NSString *)hostName port:(int)port nickname:(NSString *)nickname userDescription:(NSString *)userDescription;

- (void)disconnect;

- (TCSocket *)socket;

- (void)sendData:(NSData *)data;
- (void)sendEvent:(IRCEvent *)event;

- (IRCEventProcessor *)eventProcessor;

/* Accessors / Configurators(?) */
- (BOOL)isConnected;

- (void)setDescription:(NSString *)aDesc;

- (IRCUser *)localUser;

- (NSString *)hostName;
- (int)port;

- (NSDictionary *)channelDictionary;
- (void)addChannel:(IRCChannel *)channel;
- (void)removeChannel:(IRCChannel *)channel;
- (void)removeChannelNamed:(NSString *)channelName;

- (NSArray *)users;
- (void)addUser:(IRCUser *)user;
- (void)removeUser:(IRCUser *)user;
- (void)removeUserNamed:(NSString *)name;

@end
