/*
 * IRCChannel.h
 * Object model of an IRC channel.
 *
 * Copyright (c) 1998 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1998.
 */

@class IRCModel, NSString;

IRC_EXPORT NSString *IRCUserVoicePrefix;
IRC_EXPORT NSString *IRCUserChannelOperatorPrefix;
IRC_EXPORT NSString *IRCUserServerOperatorPrefix;

IRC_EXPORT NSString *IRCUserDidChangeNotification;
IRC_EXPORT NSString *IRCUserDidChangeNicknameNotification;

@interface IRCUser : IRCModel <NSCopying>
{
    IRCServer *_server;
    
    NSString *_nickname;
    NSString *_userName;
    NSString *_hostName;
    NSString *_userDescription;
    NSString *_serverName;
    NSString *_serverInfo;
    NSMutableArray *_channelNames;
    NSString *_awayMessage;
    NSString *_modes;

    NSCalendarDate *_idleSinceDate;
    NSCalendarDate *_signOnDate;

    BOOL _isServerOperator;
}

+ (IRCUser *)userWithNickname:(NSString *)name server:(IRCServer *)server;

- (id)initWithNickname:(NSString *)name server:(IRCServer *)server;

- (IRCServer *)server;

- (NSString *)nickname;
- (void)setNickname:(NSString *)nickname;

- (NSString *)userName;
- (NSString *)hostName;

- (NSString *)userDescription;
- (void)setUserDescription:(NSString *)userDescription;

- (NSString *)serverName;
- (NSString *)serverInfo;

- (NSArray *)channelNames;

- (BOOL)isAway;
- (NSString *)awayMessage;

- (NSString *)modes;

- (NSCalendarDate *)idleSinceDate;
- (NSCalendarDate *)signOnDate;

- (BOOL)isServerOperator;

@end
