#import "JediKnight.h"

@implementation JKApprentice

- (id)initWithMaster:(JKMaster *)master bundle:(NSBundle *)bundle
{
    if (self = [super init]) {
        _master = [master retain];
        _bundle = [bundle retain];
        _imageCache = [[NSMutableDictionary alloc] init];
    }

    return self;
}

- (void)dealloc
{
    [_master release];
    [_bundle release];
    [_imageCache release];
    
    [super dealloc];
}

- (id)copyWithZone:(NSZone *)zone
{
    return [self retain];
}

- (NSMenuItem *)registerApprenticesMenuItemTitle:(NSString *)title target:(id)target action:(SEL)action keyEquivalent:(NSString *)keyEquiv
{
    NSMenu *menu = [[self master] apprenticesMenu];
    NSMenuItem *item = [menu addItemWithTitle:title
                                       action:action
                                keyEquivalent:keyEquiv];

    [item setTarget:target];
    [menu update];

    return item;
}

- (void)finishLoading
{
}

- (void)cleanupBeforeUnloading
{
}

- (NSImage *)imageNamed:(NSString *)imageName
{
    NSEnumerator *imageTypeEnumerator = [[NSImage imageFileTypes] objectEnumerator];
    NSString *imageType = nil;

    if ([_imageCache objectForKey:imageName] == nil) {
        while (imageType = [imageTypeEnumerator nextObject]) {
            NSString *path = [[NSBundle bundleForClass:[self class]] pathForResource:imageName
                                                                              ofType:imageType];
            if (path != nil) {
                [_imageCache setObject:[[[NSImage alloc] initWithContentsOfFile:path] autorelease]
                                forKey:imageName];
                break;
            }
        }
    }

    return [_imageCache objectForKey:imageName];
}

- (JKMaster *)master
{
    return _master;
}

- (NSBundle *)bundle
{
    return _bundle;
}

@end
