#import "JediKnight.h"

DEFINE_NSSTRING(JKChannelDocumentDidCreateNotification);
DEFINE_NSSTRING(JKChannelDocumentDidBecomeMainNotification);
DEFINE_NSSTRING(JKChannelDocumentWillCloseNotification);

@implementation JKChannelDocument

+ (JKChannelDocument *)channelDocumentWithChannel:(IRCChannel *)channel
{
    return [[[self alloc] initWithChannel:channel] autorelease];
}

- (id)initWithChannel:(IRCChannel *)channel
{
    if (self = [super initWithServer:[channel server] target:channel]) {
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(channelDidChange:)
                                                     name:IRCChannelDidChangeNotification
                                                   object:_target];

        [window setTitle:[_target name]];
        [_outputManager appendClientMessage:@"Joined %@ at %@",
            [_target name],
            [[NSDate date] descriptionWithCalendarFormat:@"%I:%M%p"
                                                timeZone:nil
                                                  locale:nil]];

        [window makeKeyAndOrderFront:nil];
    }

    return self;
}

- (void)channelDidChange:(NSNotification *)notification
{
    [window setTitle:[NSString stringWithFormat:@"%@: \"%@\"", [_target name], [_target topic]]];
}

- (IRCChannel *)channel
{
    return _target;
}

- (void)orderFrontBanPanel:(id)sender
{
    [[JKInspector sharedInspector] orderFrontBanPanel:nil];
}

- (void)orderFrontSetTopicPanel:(id)sender
{
    [[JKInspector sharedInspector] orderFrontSetTopicPanel:nil];
}


- (void)serverDidPRIVMSG:(IRCServer *)server event:(IRCEvent *)event
{
    if ([_server isEqual:server]) {
        if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]]) {
            NSString *message = [[event arguments] objectAtIndex:1];

            if (![message hasPrefix:IRCCTCPDelimiterCharacter]) {
                [_outputManager appendMessage:message speaker:[[event prefix] nicknameComponent]];
                [self textViewUpdateFinished];
            }
        }
    }
}

- (void)serverDidNOTICE:(IRCServer *)server event:(IRCEvent *)event
{
    if ([_server isEqual:server]) {
        if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]]) {
            NSString *message = [[event arguments] objectAtIndex:1];

            if (![message hasPrefix:IRCCTCPDelimiterCharacter]) {
                [_outputManager appendNotice:message speaker:[[event prefix] nicknameComponent]];
                [self textViewUpdateFinished];
            }
        }
    }
}

- (void)serverDidCTCP:(IRCServer *)server event:(IRCEvent *)event
{
    if ([_server isEqual:server]) {
        if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]]) {

            if ([[[event arguments] objectAtIndex:1] isEqualToString:@"ACTION"]) {
                [_outputManager appendAction:[[event arguments] objectAtIndex:2]
                                       actor:[[event prefix] nicknameComponent]];
                [self textViewUpdateFinished];
            }
        }
    }
}

- (void)serverDidJOIN:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]])
        [_outputManager appendClientMessage:@"%@ joined %@ at %@",
            [[event prefix] nicknameComponent],
            [_target name],
            [[NSDate date] descriptionWithCalendarFormat:@"%I:%M%p"
                                                timeZone:nil
                                                  locale:nil]];
}

- (void)serverDidKICK:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]])
        [_outputManager appendClientMessage:@"%@ was kicked from %@ by %@: \"%@\" at %@",
            [[event prefix] nicknameComponent],
            [_target name],
            [[event arguments] objectAtIndex:1],
            [[event arguments] objectAtIndex:2],
            [[NSDate date] descriptionWithCalendarFormat:@"%I:%M%p"
                                                timeZone:nil
                                                  locale:nil]];
}

- (void)serverDidPART:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]])
        [_outputManager appendClientMessage:@"%@ left %@ at %@",
            [[event prefix] nicknameComponent],
            [_target name],
            [[NSDate date] descriptionWithCalendarFormat:@"%I:%M%p"
                                                timeZone:nil
                                                  locale:nil]];
}

- (void)serverDidQUIT:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[_target users] containsUserNamed:[[event prefix] nicknameComponent]])
        [_outputManager appendClientMessage:@"%@ signed off: \"%@\" at %@",
            [[event prefix] nicknameComponent],
            [[event arguments] objectAtIndex:0],
            [[NSDate date] descriptionWithCalendarFormat:@"%I:%M%p"
                                                timeZone:nil
                                                  locale:nil]];
}

- (void)serverDidTOPIC:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]])
        [_outputManager appendClientMessage:@"%@ changed the topic on %@ to \"%@\" at %@",
            [[event prefix] nicknameComponent],
            [_target name],
            [[event arguments] objectAtIndex:1],
            [[NSDate date] descriptionWithCalendarFormat:@"%I:%M%p"
                                                timeZone:nil
                                                  locale:nil]];
}

- (void)serverDidMODE:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[[event arguments] objectAtIndex:0] isEqualToStringIgnoringCase:[_target name]])
        [_outputManager appendClientMessage:@"Mode change \"%@\" on channel %@ by %@ at %@",
            [[[event arguments] subarrayWithRange:NSMakeRange(1, [[event arguments] count]-1)] componentsJoinedByString:@" "],
            [_target name],
            [[event prefix] nicknameComponent],
            [[NSDate date] descriptionWithCalendarFormat:@"%I:%M%p"
                                                timeZone:nil
                                                  locale:nil]];
}

- (void)serverDidNICK:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[_target users] containsUserNamed:[[event prefix] nicknameComponent]])
        [_outputManager appendClientMessage:@"%@ is now known as %@",
            [[event prefix] nicknameComponent],
            [[event arguments] objectAtIndex:0]];
}

- (void)windowWillClose:(NSNotification *)notification
{
    [_server sendEvent:[IRCEvent partEventWithTarget:[_target name]]];

    [super windowWillClose:notification];
}

@end
