#import "JediKnight.h"

DEFINE_NSSTRING(JKChannelManagerItem);

@implementation JKChannelManager

- (id)initWithServer:(IRCServer *)server
{
    NSArray *columns;
    
    if (self = [super init]) {
        _server = server;

       [panel setTitle:[NSString stringWithFormat:TCLocalizedStringForKey(@"channelManagerTitleFormat"),
            [_server hostName]]];

        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(channelsDidChange:)
                                                     name:IRCServerChannelListDidChangeNotification
                                                   object:_server];

        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(tableViewSelectionIsChanging:)
                                                     name:NSTableViewSelectionIsChangingNotification
                                                   object:tableView];

        [tableView setTarget:self];
        [tableView setDoubleAction:@selector(tableViewDoubleClicked:)];
        
        columns = [tableView tableColumns];
        [[columns objectAtIndex:0] setEditable:NO];
        [[columns objectAtIndex:1] setEditable:NO];
        [[columns objectAtIndex:2] setEditable:NO];
        [[columns objectAtIndex:0] setIdentifier:@"Channel"];
        [[columns objectAtIndex:1] setIdentifier:@"Users"];
        [[columns objectAtIndex:2] setIdentifier:@"Topic"];
    }

    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (NSArray *)selectedChannelNames
{
    NSEnumerator *rowEnumerator = [tableView selectedRowEnumerator];
    NSNumber *rowNumber;
    NSMutableArray *array = [NSMutableArray array];

    while (rowNumber = [rowEnumerator nextObject]) {
        NSString *channel;

        channel = [[tableView dataSource] tableView:tableView
                          objectValueForTableColumn:[tableView tableColumnWithIdentifier:@"Channel"]
                                                row:[rowNumber intValue]];
        [array addObject:channel];
    }

    return array;
}

- (void)tableViewDoubleClicked:(id)sender
{
    NSEnumerator *enumerator = [[self selectedChannelNames] objectEnumerator];
    NSString *channel;

    while (channel = [enumerator nextObject])
        [_server sendEvent:[IRCEvent joinEventWithChannel:channel key:nil]];
}

- (void)tableViewSelectionIsChanging:(NSNotification *)notification
{
    BOOL enabledFlag = [tableView numberOfSelectedRows];

    [[matrix cellAtRow:0 column:1] setEnabled:enabledFlag];
    [[matrix cellAtRow:0 column:2] setEnabled:enabledFlag];
}

- (void)addChannelEntry:(id)sender
{
    [addPanel makeKeyAndOrderFront:nil];
    if ([NSApp runModalForWindow:addPanel] == NSRunStoppedResponse) {
        IRCChannel *channel = nil;
        NSString *channelName = [addPanelNameField stringValue];

        if (![channelName hasPrefix:@"#"])
            channelName = [channelName stringByPrependingString:@"#"];

        channel = [IRCChannel channelWithName:channelName server:_server];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(channelsDidChange:)
                                                     name:IRCChannelDidChangeNotification
                                                   object:channel];

        [_server addChannel:channel];
    }
    [addPanel orderOut:nil];
}

- (void)channelEntryAdded:(id)sender
{
    [NSApp stopModal];
}

- (void)channelEntryCancelled:(id)sender
{
    [NSApp abortModal];
}

- (void)removeSelectedChannelEntry:(id)sender
{
    NSEnumerator *enumerator = [[self selectedChannelNames] objectEnumerator];
    NSString *channel;

    while (channel = [enumerator nextObject])
        [_server removeChannelNamed:channel];
}

- (void)retrieveListOfChannels:(id)sender
{
}

// Notification methods
- (void)channelsDidChange:(NSNotification *)notification
{
    [tableView reloadData];
}

// NSTableViewDataSource protocol
- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)column row:(int)row
{
    NSArray *array;

    array = [[[_server channelDictionary] allValues] sortedArrayUsingSelector:@selector(compare:)];
    
    if ([[column identifier] isEqualToString:@"Channel"])
        return [[array objectAtIndex:row] name];
    
    else if ([[column identifier] isEqualToString:@"Users"])
        return [NSString stringWithFormat:@"%d",
            [[[array objectAtIndex:row] users] count]];
    
    else if ([[column identifier] isEqualToString:@"Topic"])
        return [[array objectAtIndex:row] topic];

    else
        TCLog(self, _cmd, @"Unknown column %@(%@)",
              [column description], [[column identifier] description]);

    return nil;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [[_server channelDictionary] count];
}

@end
