#import "JediKnight.h"
#import "JKConsole.h"

@implementation JKConsole

- (void)finishLoading
{
    if (![NSBundle loadNibNamed:@"JKConsole" owner:self])
        TCLog(self, _cmd, @"Cannot load JKConsole.nib; apprentice may be damaged.");

    [self registerApprenticesMenuItemTitle:@"Console"
                                    target:panel
                                    action:@selector(makeKeyAndOrderFront:)
                             keyEquivalent:@"C"];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(serverDidConnect:)
                                                 name:IRCServerDidConnectNotification
                                               object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(serverWillTerminate:)
                                                 name:IRCServerWillTerminateNotification
                                               object:nil];

    [panel setFrameAutosaveName:@"JKConsolePanel"];
    [panel setFrameUsingName:@"JKConsolePanel"];

    [radioMatrix selectCellWithTag:[[NSUserDefaults standardUserDefaults] integerForKey:@"JKConsoleRadioMatrixSelection"]];
    [textView setSelectedRange:NSMakeRange(0, 0)];
}

- (void)clearTextView:(id)sender
{
    [textView clear];
}

- (void)radioMatrixSelectionDidChange:(id)sender
{
    [[NSUserDefaults standardUserDefaults] setInteger:[[radioMatrix selectedCell] tag]
                                               forKey:@"JKConsoleRadioMatrixSelection"];
}

- (void)serverDidConnect:(NSNotification *)notification
{
    IRCServer *server = [notification object];

    [[server eventProcessor] addResponder:self];
    [textView appendWithFormat:@"--- Connected to %@ ---\n", [server hostName]];
}

- (void)serverWillTerminate:(NSNotification *)notification
{
    IRCServer *server = [notification object];

    [textView appendWithFormat:@"--- Disconnected from %@ ---\n", [server hostName]];
    [[server eventProcessor] removeResponder:self];
}

- (BOOL)server:(IRCServer *)server willProcessEvent:(IRCEvent *)event
{
    if ([[radioMatrix selectedCell] tag] == 0) {
        NSString *line = [[event arguments] componentsJoinedByString:@" "];
        [textView appendWithFormat:@"%@\n", line];
    }
    else
        [textView appendWithFormat:@"%@\n", [event description]];

    [textView scrollRangeToVisible:NSMakeRange([[textView string] length], 0)];

    return NO;
}

- (void)cleanupBeforeUnloading
{
}

- (NSTextView *)textView
{
    return textView;
}

- (void)orderFrontFindPanel:(id)sender
{
    [[JKFindPanel sharedInstance] orderFrontFindPanel:sender];
}

- (void)findNext:(id)sender
{
    [[JKFindPanel sharedInstance] findNext:sender];
}

- (void)findPrevious:(id)sender
{
    [[JKFindPanel sharedInstance] findPrevious:sender];
}

- (void)enterSelection:(id)sender
{
    NSRange range = [textView selectedRange];
    if (range.length) {
        [[JKFindPanel sharedInstance] setFindString:[[textView string] substringWithRange:range]];
    } else {
        NSBeep();
    }
}

- (void)jumpToSelection:(id)sender
{
    [textView scrollRangeToVisible:[textView selectedRange]];
}

@end
