#import "JediKnight.h"

NSString *JKInputManagerDefaultCommandCharacter = @"/";

@implementation JKInputManager

+ (JKInputManager *)defaultInputManager
{
    static id singleton = nil;

    if (singleton == nil)
        singleton = [[self alloc] init];

    return singleton;
}

- (id)init
{
    if (self = [super init]) {
        _commandCharacter = [JKInputManagerDefaultCommandCharacter retain];
        _invocationDictionary = [[NSMutableDictionary alloc] init];
        _privacyDictionary = [[NSMutableDictionary alloc] init];
    }

    return self;
}

- (NSString *)commandCharacter
{
    return _commandCharacter;
}

- (void)setCommandCharacter:(NSString *)commandCharacter
{
    [_commandCharacter release];
    _commandCharacter = [commandCharacter retain];
}

- (JKInterpretationResult)interpretCommand:(NSString *)aString privateOwner:(id)anObject
{
    if ([aString hasPrefix:_commandCharacter]) {
        NSScanner *scanner = [NSScanner scannerWithString:aString];
        NSString *command, *parameters;

        [scanner scanString:JKInputManagerDefaultCommandCharacter intoString:NULL];
        if ([scanner isAtEnd]) return JKInterpretedInvalidCommand;
        [scanner scanUpToString:@" " intoString:&command];
        if ([scanner isAtEnd]) return JKInterpretedInvalidCommand;
        [scanner scanString:@" " intoString:NULL];
        if ([scanner isAtEnd]) return JKInterpretedInvalidCommand;
        [scanner scanUpToString:@"" intoString:&parameters];

        if ([_invocationDictionary objectForKey:command] != nil) {
            NSInvocation *invocation = [_invocationDictionary objectForKey:command];

            [invocation setArgument:&parameters atIndex:2];
            if ([[_privacyDictionary objectForKey:command] isEqualToString:@"YES"])
                [invocation invokeWithTarget:anObject];
            else
                [invocation invoke];

            return JKInterpretedCommand;
        }
        else
            return JKInterpretedInvalidCommand;
    }

    return JKInterpretedNoCommand;
}

- (void)registerCommandName:(NSString *)commandName target:(id)anObject selector:(SEL)selector isPrivate:(BOOL)privateFlag
{
    NSMethodSignature *signature = [anObject methodSignatureForSelector:selector];
    NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];

    [invocation setTarget:anObject];
    [invocation setSelector:selector];

    [_invocationDictionary setObject:invocation forKey:commandName];
    [_privacyDictionary setObject:(privateFlag ? @"YES" : @"NO") forKey:commandName];
}

@end
