#import "JediKnight.h"

@implementation JKInspector

+ (JKInspector *)sharedInspector
{
    return (JKInspector *)[self sharedPanelController];
}

- (id)init
{
    if (self = [super init]) {
        [usersViewContainer retain];		// retain switchable views
        [informationViewContainer retain];
        [banListViewContainer retain];

        [usersViewContainer setFrame:[switchView frame]];
        [informationViewContainer setFrame:[switchView frame]];
        [banListViewContainer setFrame:[switchView frame]];

        [[panel contentView] replaceSubview:switchView with:usersViewContainer];
        switchView = usersViewContainer;

        [tableView setTarget:self];
        [tableView setDoubleAction:@selector(openPrivateConversation:)];
        [tableView setAllowsMultipleSelection:NO];
        [tableView setAllowsColumnResizing:YES];
        [tableView setAllowsColumnReordering:YES];

        _userIsNothingImage = [NSImage imageNamed:@"userIsNothingCellIcon"];
        _userIsChannelOperatorImage = [NSImage imageNamed:@"userIsChannelOperatorCellIcon"];
        _userIsVoicedImage = [NSImage imageNamed:@"userIsVoicedCellIcon"];

        [[tableView tableColumnWithIdentifier:@"Image"] setDataCell:[[NSImageCell alloc] initImageCell:nil]];

        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(documentDidBecomeMain:)
                                                     name:JKChannelDocumentDidBecomeMainNotification
                                                   object:nil];
    }

    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    [usersViewContainer release];
    [informationViewContainer release];
    [banListViewContainer release];
    
    [super dealloc];
}

- (void)switchViews:(id)sender
{
    switch ([[sender selectedItem] tag]) {
        case 0:
            if (switchView != usersViewContainer) {
                [[panel contentView] replaceSubview:switchView with:usersViewContainer];
                [usersViewContainer setFrame:[switchView frame]];
                switchView = usersViewContainer;
            }
            break;
            
        case 1:
            if (switchView != informationViewContainer) {
                [[panel contentView] replaceSubview:switchView with:informationViewContainer];
                [informationViewContainer setFrame:[switchView frame]];
                switchView = informationViewContainer;
            }
            break;

        case 2:
            if (switchView != banListViewContainer) {
                [[panel contentView] replaceSubview:switchView with:banListViewContainer];
                [banListViewContainer setFrame:[switchView frame]];
                switchView = banListViewContainer;
            }
            break;
            
        default:
            [NSException raise:NSInvalidArgumentException
                        format:@"broken interface -- tag not 0, 1 or 2"];
    }
}

- (void)documentDidBecomeMain:(NSNotification *)notification
{
    if (_inspectedChannel != nil)
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:IRCChannelDidChangeNotification
                                                      object:_inspectedChannel];

    _inspectedChannel = [[notification object] channel];
    if (_inspectedChannel != nil) {
        [panel setTitle:[NSString stringWithFormat:@"Inspecting %@", [_inspectedChannel name]]];
        [banPanel setTitle:[NSString stringWithFormat:@"Add Ban: %@", [_inspectedChannel name]]];
        [setTopicPanel setTitle:[NSString stringWithFormat:@"Set Topic: %@", [_inspectedChannel name]]];
       
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(inspectedChannelDidChange:)
                                                     name:IRCChannelDidChangeNotification
                                                   object:_inspectedChannel];
    }
    else
        [panel setTitle:@"No Inspector"];

    [self inspectedChannelDidChange:nil];
}

- (void)inspectedChannelDidChange:(NSNotification *)notification
{
    [tableView reloadData];
    [banBrowser reloadColumn:0];

    [serverTextField setStringValue:IRCEmptyStringIfNil([[_inspectedChannel server] hostName])];
    [topicSetByTextField setStringValue:IRCEmptyStringIfNil([_inspectedChannel topicSetByUserName])];
    [topicSetAtTextField setStringValue:IRCEmptyStringIfNil([[_inspectedChannel topicDate] description])];

    [[flagsForm cellAtIndex:0] setIntValue:[_inspectedChannel userLimit]];
    [[flagsForm cellAtIndex:1] setStringValue:IRCEmptyStringIfNil([_inspectedChannel channelKey])];

    [flagsMatrix setState:[_inspectedChannel isPrivate] atRow:0 column:0];
    [flagsMatrix setState:[_inspectedChannel isSecret] atRow:1 column:0];
    [flagsMatrix setState:[_inspectedChannel isModerated] atRow:2 column:0];
    [flagsMatrix setState:[_inspectedChannel isInviteOnly] atRow:3 column:0];
    [flagsMatrix setState:[_inspectedChannel isTopicOperatorOnly] atRow:4 column:0];
    [flagsMatrix setState:[_inspectedChannel isClosedToExternalMessages] atRow:5 column:0];
    [flagsMatrix setState:[_inspectedChannel isKeyNeeded] atRow:6 column:0];
    [flagsMatrix setState:[_inspectedChannel isUserLimited] atRow:7 column:0];

    [flagsMatrix setEnabled:[_inspectedChannel containsOperator:[[_inspectedChannel server] localUser]]];
    [flagsForm setEnabled:[_inspectedChannel containsOperator:[[_inspectedChannel server] localUser]]];

    [orderFrontBanPanelButton setEnabled:[_inspectedChannel containsOperator:[[_inspectedChannel server] localUser]]];
    [addBanButton setEnabled:[_inspectedChannel containsOperator:[[_inspectedChannel server] localUser]]];
    [setTopicButton setEnabled:[_inspectedChannel containsOperator:[[_inspectedChannel server] localUser]]];
    [removeBanButton setEnabled:([_inspectedChannel containsOperator:[[_inspectedChannel server] localUser]] &&
                                 [banBrowser selectedRowInColumn:0] != -1)];

    [setTopicField setStringValue:IRCEmptyStringIfNil([_inspectedChannel topic])];

    // insane logic to get the user info section to work
    if ([[_inspectedChannel users] count] > 0) {
        if (_selectedUser == nil) {
            [tableView selectRow:0 byExtendingSelection:NO];
            _selectedUser = [[_inspectedChannel users] objectAtIndex:0];
        }

        if ([tableView selectedRow] != -1)
            [self inspectedUserDidChange:tableView];

        [tableView selectRow:[[_inspectedChannel users] indexOfObject:_selectedUser] byExtendingSelection:NO];
    }
}

- (void)inspectedUserDidChange:(id)sender
{
    _selectedUser = [[_inspectedChannel users] objectAtIndex:[sender selectedRow]];

    [[userInfoMatrix cellAtRow:0 column:0] setStringValue:IRCEmptyStringIfNil([_selectedUser userDescription])];
    [[userInfoMatrix cellAtRow:1 column:0] setStringValue:IRCEmptyStringIfNil([_selectedUser serverName])];
    [[userInfoMatrix cellAtRow:2 column:0] setStringValue:IRCEmptyStringIfNil([_selectedUser serverInfo])];

    [[userInfoMatrix cellAtRow:3 column:0] setStringValue:
        ([_selectedUser awayMessage] ? [_selectedUser awayMessage] : @"Not available")];;
    
    [[userInfoMatrix cellAtRow:4 column:0] setStringValue:
        ([_selectedUser idleSinceDate] ?
         [[_selectedUser idleSinceDate] description] :
         @"Not available")];

    [[userInfoMatrix cellAtRow:5 column:0] setStringValue:
        ([_selectedUser signOnDate] ?
         [[_selectedUser signOnDate] description] :
         @"Not available")];
}

- (void)_sendEventWithControlState:(BOOL)state character:(NSString *)character
{
    [[_inspectedChannel server] sendEvent:
        [IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                modes:[NSArray arrayWithObjects:(state ? @"+" : @"-"), character, nil]
                            arguments:[NSArray array]]];
}

- (void)channelModeBoxesDidChange:(id)sender
{
    [applyButton setEnabled:YES];
}

- (void)applyChannelModeChanges:(id)sender
{
    NSMutableArray *modes = [NSMutableArray array];
    NSMutableArray *arguments = [NSMutableArray array];
    
    if ([[flagsMatrix cellAtRow:0 column:0] state] != [_inspectedChannel isPrivate])
        [[flagsMatrix cellAtRow:0 column:0] state] ? [modes addObject:@"+p"] : [modes addObject:@"-p"];
    if ([[flagsMatrix cellAtRow:1 column:0] state] != [_inspectedChannel isSecret])
        [[flagsMatrix cellAtRow:1 column:0] state] ? [modes addObject:@"+s"] : [modes addObject:@"-s"];
    if ([[flagsMatrix cellAtRow:2 column:0] state] != [_inspectedChannel isModerated])
        [[flagsMatrix cellAtRow:2 column:0] state] ? [modes addObject:@"+m"] : [modes addObject:@"-m"];
    if ([[flagsMatrix cellAtRow:3 column:0] state] != [_inspectedChannel isInviteOnly])
        [[flagsMatrix cellAtRow:3 column:0] state] ? [modes addObject:@"+i"] : [modes addObject:@"-i"];
    if ([[flagsMatrix cellAtRow:4 column:0] state] != [_inspectedChannel isTopicOperatorOnly])
        [[flagsMatrix cellAtRow:4 column:0] state] ? [modes addObject:@"+t"] : [modes addObject:@"-t"];
    if ([[flagsMatrix cellAtRow:5 column:0] state] != [_inspectedChannel isClosedToExternalMessages])
        [[flagsMatrix cellAtRow:5 column:0] state] ? [modes addObject:@"+n"] : [modes addObject:@"-n"];

    if ([[flagsMatrix cellAtRow:6 column:0] state] != [_inspectedChannel isKeyNeeded] ||
        !([[[flagsForm cellAtIndex:1] stringValue] isEqualToStringIgnoringCase:[_inspectedChannel channelKey]])) {
        [[flagsMatrix cellAtRow:6 column:0] state] ? [modes addObject:@"+k"] : [modes addObject:@"-k"];
        [arguments addObject:[[flagsForm cellAtIndex:1] stringValue]];
    }
    if ([[flagsMatrix cellAtRow:7 column:0] state] != [_inspectedChannel isUserLimited] ||
        [[flagsForm cellAtIndex:0] intValue] != [_inspectedChannel userLimit]) {
        if ([[flagsMatrix cellAtRow:7 column:0] state]) {
            [modes addObject:@"+l"];
            [arguments addObject:[[flagsForm cellAtIndex:0] stringValue]];
        }
        else
            [modes addObject:@"-l"];
    }

    [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                  modes:modes
                                                              arguments:arguments]];

    [applyButton setEnabled:NO];
}

- (void)orderFrontBanPanel:(id)sender
{
    [banPanel makeKeyAndOrderFront:nil];
}

- (void)orderFrontSetTopicPanel:(id)sender
{
    [setTopicPanel makeKeyAndOrderFront:nil];
}

- (void)banPanelAddButtonClicked:(id)sender
{
    [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                  modes:[NSArray arrayWithObject:@"+b"]
                                                              arguments:[NSArray arrayWithObject:[addBanField stringValue]]]];
    
    [[addBanField window] makeFirstResponder:addBanField];
}

- (void)setTopicButtonClicked:(id)sender
{
    [[_inspectedChannel server] sendEvent:[IRCEvent topicEventWithChannel:[_inspectedChannel name]
                                                                    topic:[setTopicField stringValue]]];;

    [[setTopicField window] makeFirstResponder:setTopicField];
}

- (void)removeSelectedBan:(id)sender
{
    NSEnumerator *selectedCellEnumerator = [[banBrowser selectedCells] objectEnumerator];
    NSCell *cell;

    while (cell = [selectedCellEnumerator nextObject]) {
        [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                      modes:[NSArray arrayWithObject:@"-b"]
                                                                  arguments:[NSArray arrayWithObject:[cell stringValue]]]];
    }
}

- (void)giveOperator:(id)sender
{
    if ([tableView selectedRow] != -1) {
        NSArray *opModes = [NSArray arrayWithObjects:@"+", @"o", nil];
        NSArray *opTarget = [NSArray arrayWithObject:[[[_inspectedChannel users] objectAtIndex:[tableView selectedRow]] name]];
        
        [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                      modes:opModes
                                                                  arguments:opTarget]];
    }
}
    
- (void)removeOperator:(id)sender
{
    if ([tableView selectedRow] != -1) {
        NSArray *opModes = [NSArray arrayWithObjects:@"-", @"o", nil];
        NSArray *opTarget = [NSArray arrayWithObject:[[[_inspectedChannel users] objectAtIndex:[tableView selectedRow]] name]];

        [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                      modes:opModes
                                                                  arguments:opTarget]];
    }
}

- (void)giveModerator:(id)sender
{
    if ([tableView selectedRow] != -1) {
        NSArray *opModes = [NSArray arrayWithObjects:@"+", @"v", nil];
        NSArray *opTarget = [NSArray arrayWithObject:[[[_inspectedChannel users] objectAtIndex:[tableView selectedRow]] name]];

        [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                      modes:opModes
                                                                  arguments:opTarget]];
    }
}

- (void)removeModerator:(id)sender
{
    if ([tableView selectedRow] != -1) {
        NSArray *opModes = [NSArray arrayWithObjects:@"-", @"v", nil];
        NSArray *opTarget = [NSArray arrayWithObject:[[[_inspectedChannel users] objectAtIndex:[tableView selectedRow]] name]];

        [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                      modes:opModes
                                                                  arguments:opTarget]];
    }
}

- (void)kickFromChannel:(id)sender
{
    if ([tableView selectedRow] != -1) {
        [[_inspectedChannel server] sendEvent:[IRCEvent kickEventWithChannel:[_inspectedChannel name]
                                                                    nickname:[[[_inspectedChannel users] objectAtIndex:[tableView selectedRow]] name]
                                                                      reason:@""]];
    }
}

// ban routine. observe the awesome power of Foundation...
- (void)banFromChannel:(id)sender;
{
    if ([tableView selectedRow] != -1) {
        IRCUser *user = [[_inspectedChannel users] objectAtIndex:[tableView selectedRow]];
        NSString *userName = [[user userName] hasPrefix:@"~"] ? [[user userName] substringFromIndex:1] : [user userName];
        NSArray *hostNameComponents = [[user hostName] componentsSeparatedByString:@"."];
        NSString *hostName = [[hostNameComponents subarrayWithRange:NSMakeRange([hostNameComponents count]-2, 2)] componentsJoinedByString:@"."];
        NSArray *opModes = [NSArray arrayWithObjects:@"+", @"b", nil];
        NSArray *opTarget = [NSArray arrayWithObject:[NSString stringWithFormat:@"*!*%@@*%@", userName, hostName]];
        
        [[_inspectedChannel server] sendEvent:[IRCEvent modeEventWithTarget:[_inspectedChannel name]
                                                                      modes:opModes
                                                                  arguments:opTarget]];
    }
}

- (void)openPrivateConversation:(id)sender
{
    if ([tableView selectedRow] != -1) {
        IRCUser *user = [[_inspectedChannel users]  objectAtIndex:[tableView selectedRow]];

        if ([[[JKMaster jediMaster] queryDocuments] objectForKey:[user nickname]] == nil)
            [[JKMaster jediMaster] addQueryDocument:[JKQueryDocument queryDocumentWithUser:user]];

        [[[[[JKMaster jediMaster] queryDocuments] objectForKey:[user nickname]] window] makeKeyAndOrderFront:nil];
    }
}

// user NSTableView delegate methods
- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)column row:(int)row
{
    IRCUser *user = [[_inspectedChannel users] objectAtIndex:row];

    if ([[column identifier] isEqualToString:@"Image"]) {
        if ([_inspectedChannel containsOperator:user])
            return _userIsChannelOperatorImage;
        else if ([_inspectedChannel containsModerator:user])
            return _userIsVoicedImage;
        else
            return _userIsNothingImage;
    }
    else if ([[column identifier] isEqualToString:@"Name"]) {
        return [user nickname];
    }
    else if ([[column identifier] isEqualToString:@"Userhost"]) {
        if ([user userName] == nil || [user hostName] == nil)
            return @"";
        else
            return [NSString stringWithFormat:@"%@@%@", [user userName] ? [user userName] : @"", [user hostName] ? [user hostName] : @""];
    }
    else
        return nil;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [[_inspectedChannel users] count];
}

// ban browser delegate methods
- (int)browser:(NSBrowser *)sender numberOfRowsInColumn:(int)column
{
    [removeBanButton setEnabled:NO];
    return [[_inspectedChannel banList] count];
}

- (void)browserSelectionDidChange:(id)sender
{
    [removeBanButton setEnabled:[_inspectedChannel containsOperator:[[_inspectedChannel server] localUser]]];
}

- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column
{
    if (column == 0) {
        [cell setStringValue:[[_inspectedChannel banList] objectAtIndex:row]];
        [cell setLeaf:YES];
        [cell setLoaded:YES];
    }
}

@end
