#import "JediKnight.h"

NSString *JKLastSavedRepresentationNameItemKey = @"JKLastSavedRepresentationNameItemKey";
NSString *JKPathComponent = @"JediKnight";
NSString *JKApprenticeExtension = @"apprentice";
NSString *JKSavedRepresentationExtension = @"jediKnight";

@implementation JKMaster

static JKMaster *_jediMaster = nil;

+ (JKMaster *)jediMaster
{
    return _jediMaster;
}

+ (void)initialize
{
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            @"irc2.blackened.com", @"JKDefaultServer",
            @"6667", @"JKDefaultPort",
            [NSUserName() copy], @"JKDefaultNick",
            [NSFullUserName() copy], @"JKDefaultInfo",
            nil]];
}

+ (NSString *)defaultServer
{
    return [[NSUserDefaults standardUserDefaults] stringForKey:@"JKDefaultServer"];
}

+ (NSString *)defaultPort
{
    return [[NSUserDefaults standardUserDefaults] stringForKey:@"JKDefaultPort"];
}

+ (NSString *)defaultNickname
{
    return [[NSUserDefaults standardUserDefaults] stringForKey:@"JKDefaultNick"];
}

+ (NSString *)defaultUserInfo
{
    return [[NSUserDefaults standardUserDefaults] stringForKey:@"JKDefaultInfo"];
}

- (void)loadApprentices
{
    NSMutableArray *searchPaths = [NSMutableArray array];
    NSMutableArray *apprentices = [NSMutableArray array];
    NSEnumerator *objectEnumerator;
    NSString *path;
    JKApprentice *apprentice;

    // first we build an array containing all the paths we'd like to search.
    [searchPaths addObject:[[NSBundle mainBundle] resourcePath]];

#if defined(__APPLE__)
    objectEnumerator = [NSSearchPathForDirectoriesInDomains(NSAllLibrariesDirectory, NSAllDomainsMask, YES) objectEnumerator];
#else
    objectEnumerator = [NSStandardLibraryPaths() objectEnumerator];
#endif
    while (path = [objectEnumerator nextObject])
        [searchPaths addObject:[path stringByAppendingPathComponent:JKPathComponent]];

    // once built, we allocate and initialize the apprentice instances
    objectEnumerator = [searchPaths objectEnumerator];    
    while (path = [objectEnumerator nextObject]) {
        NSDirectoryEnumerator *directoryEnumerator;
        NSString *bundleName;

        directoryEnumerator = [[NSFileManager defaultManager] enumeratorAtPath:path];

        while (bundleName = [directoryEnumerator nextObject]) {
            if ([[bundleName pathExtension] isEqualToString:JKApprenticeExtension]) {
                NSString *bundlePath = [path stringByAppendingPathComponent:bundleName];
                NSBundle *newBundle = [NSBundle bundleWithPath:bundlePath];
                JKApprentice *newInstance = [[newBundle principalClass] alloc];

                if (newInstance != nil) {
                    [newInstance initWithMaster:self bundle:newBundle];
                    [newInstance autorelease];

                    [apprentices addObject:newInstance];
                }
                else
                    NSRunAlertPanel(TCLocalizedStringForKey(@"apprenticeError"),
                                    TCLocalizedStringForKey(@"apprenticeErrorFormat"),
                                    TCLocalizedStringForKey(@"cancel"), nil, nil,
                                    [bundleName lastPathComponent]);
                
                [directoryEnumerator skipDescendents];
            }
        }
    }

    // once all bundles are loaded, we send each instance the -finishLoading message
    _apprentices = [apprentices copy];
    objectEnumerator = [_apprentices objectEnumerator];
    while (apprentice = [objectEnumerator nextObject])
        [apprentice finishLoading];
}

- (id)init
{
    if (self = [super init]) {
        _jediMaster = self;
        
        _channelDocuments = [[NSMutableDictionary alloc] init];
        _queryDocuments = [[NSMutableDictionary alloc] init];
    }

    return self;
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
    IRCSetApplicationObject(self);
    
    [[openPanelForm cellAtRow:0 column:0] setStringValue:[JKMaster defaultServer]];
    [[openPanelForm cellAtRow:1 column:0] setStringValue:[JKMaster defaultPort]];
    [[openPanelForm cellAtRow:2 column:0] setStringValue:[JKMaster defaultNickname]];
    [[openPanelForm cellAtRow:3 column:0] setStringValue:[JKMaster defaultUserInfo]];

    [addBanMenuItem setAction:@selector(orderFrontBanPanel:)];
    [setTopicMenuItem setAction:@selector(orderFrontSetTopicPanel:)];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(serverDidConnect:)
                                                 name:IRCServerDidConnectNotification
                                               object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(serverWillTerminate:)
                                                 name:IRCServerWillTerminateNotification
                                               object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(channelDocumentWillClose:)
                                                 name:JKChannelDocumentWillCloseNotification
                                               object:nil];

    [JKInspector sharedInspector];	// load the inspector so it can pay attention
    [self loadApprentices];
}

- (void)connectionPanelOk:(id)sender
{
    NSString *hostName = [[openPanelForm cellAtRow:0 column:0] stringValue];
    int port = [[openPanelForm cellAtRow:1 column:0] intValue];
    NSString *nickname = [[openPanelForm cellAtRow:2 column:0] stringValue];
    NSString *userDescription = [[openPanelForm cellAtRow:3 column:0] stringValue];

    [[NSUserDefaults standardUserDefaults] setObject:hostName forKey:@"JKDefaultServer"];
    [[NSUserDefaults standardUserDefaults] setInteger:port forKey:@"JKDefaultPort"];
    [[NSUserDefaults standardUserDefaults] setObject:nickname forKey:@"JKDefaultNick"];
    [[NSUserDefaults standardUserDefaults] setObject:userDescription forKey:@"JKDefaultInfo"];

    NS_DURING {
        _server = [[IRCServer alloc] initWithHostName:hostName
                                                 port:port
                                             nickname:nickname
                                      userDescription:userDescription];
    }
    NS_HANDLER {
        TCShowModelessAlertPanel(TCLocalizedStringForKey(@"connectionError"),
                                 TCLocalizedStringForKey(@"connectionErrorFormat"),
                                 [_server hostName], [localException reason]);
    }
    NS_ENDHANDLER;

    [[sender window] orderOut:nil];
}

- (void)openRepresentation:(id)sender
{
    NSOpenPanel *openPanel = [NSOpenPanel openPanel];
    NSString *lastDirectory = [[NSUserDefaults standardUserDefaults] stringForKey:@"JKLastOpenedDirectory"];
    NSString *lastFile = [[NSUserDefaults standardUserDefaults] stringForKey:@"JKLastOpenedFile"];

    if (lastDirectory == nil)
        lastDirectory = NSHomeDirectory();

    [openPanel setAllowsMultipleSelection:NO];
    [openPanel setRequiredFileType:JKSavedRepresentationExtension];
    [openPanel setTitle:TCLocalizedStringForKey(@"openConnection")];

    if ([openPanel runModalForDirectory:lastDirectory file:lastFile] == NSOKButton) {
        JKPersistentServerRepresentation *representation;

        representation = [NSUnarchiver unarchiveObjectWithFile:[openPanel filename]];
        [[NSUserDefaults standardUserDefaults] setObject:[openPanel directory]
                                                  forKey:@"JKLastOpenedDirectory"];
        [[NSUserDefaults standardUserDefaults] setObject:[[[openPanel filename] lastPathComponent] retain]
                                                  forKey:@"JKLastOpenedFile"];

        NS_DURING {
            NSEnumerator *channelEnumerator = [[representation knownChannels] objectEnumerator];
            NSString *channelName;

            _server = [[IRCServer alloc] initWithHostName:[representation hostName]
                                                     port:[representation port]
                                                 nickname:[representation nickname]
                                          userDescription:[representation userDescription]];

            while (channelName = [channelEnumerator nextObject])
                [_server addChannel:[IRCChannel channelWithName:channelName server:_server]];

            [[_server metaData] setObject:[openPanel filename] forKey:JKLastSavedRepresentationNameItemKey];
        }
        NS_HANDLER {
            TCShowModelessAlertPanel(TCLocalizedStringForKey(@"connectionError"),
                                     TCLocalizedStringForKey(@"connectionErrorFormat"),
                                     [_server hostName], [localException reason]);
        }
        NS_ENDHANDLER;
    }
}

- (void)saveRepresentation:(id)sender
{
    JKPersistentServerRepresentation *representation;

    representation = [JKPersistentServerRepresentation serverRepresentationWithServer:_server];
    [NSArchiver archiveRootObject:representation
                           toFile:[[_server metaData] objectForKey:JKLastSavedRepresentationNameItemKey]];
}

- (void)saveRepresentationAs:(id)sender
{
    NSSavePanel *savePanel = [NSSavePanel savePanel];
    NSString *lastDirectory = [[NSUserDefaults standardUserDefaults] stringForKey:@"JKLastSavedDirectory"];

    if (lastDirectory == nil)
        lastDirectory = NSHomeDirectory();

    [savePanel setRequiredFileType:@"jediKnight"];
    [savePanel setTitle:TCLocalizedStringForKey(@"saveConnection")];

    if ([savePanel runModalForDirectory:lastDirectory
                                   file:[NSString stringWithFormat:@"%@:%d",
                                       [_server hostName], [_server port]]] == NSOKButton) {
        [[NSUserDefaults standardUserDefaults] setObject:[savePanel directory]
                                                  forKey:@"JKLastSavedDirectory"];
        [[_server metaData] setObject:[savePanel filename] forKey:JKLastSavedRepresentationNameItemKey];
        [self saveRepresentation:nil];
    }
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    // validate the menu items based on the system status.
    // used to use menu item titles. bad! it's even in the NeXT documentation!
    // not localizable! bad!
    SEL action = [anItem action];		// cache the selector, for speed (pfft)

    // these items only work when the system is disconnected.
    if (action == @selector(orderFrontNewConnectionPanel:) ||
        action == @selector(openRepresentation:))
       return (_server == nil);

    // these items only work when the system is connected.
    if (action == @selector(saveRepresentationAs:) ||
        action == @selector(disconnect:) ||
        action == @selector(orderFrontInspector:) ||
        action == @selector(orderFrontNickChangePanel:))
        return (_server != nil);

    // save only works if a file name has been set;
    if (action == @selector(saveRepresentation:))
        return ([[_server metaData] objectForKey:JKLastSavedRepresentationNameItemKey] != nil);

    // the channels panel can only be ordered front when we're connected & the panel has been created.
    if (action == @selector(orderFrontChannelsPanel:))
        return ([[_server metaData] objectForKey:JKChannelManagerItem] != nil);

    return YES;
}

- (id)windowWillReturnFieldEditor:(NSWindow *)sender toObject:(id)anObject
{
    // set the help text for each field. got to hold the newbie's hands, you know..
    if (anObject == openPanelForm) {
        NSString *key = [NSString stringWithFormat:@"newConnectionField%dHelpKey",
            [openPanelForm indexOfSelectedItem]];
        [helpTextField setStringValue:TCLocalizedStringForKey(key)];
    }
    return nil;
}

- (void)disconnect:(id)sender
{
    [_server disconnect];
}

- (void)orderFrontNewConnectionPanel:(id)sender
{
    [[openPanelForm window] makeKeyAndOrderFront:sender];
}

- (void)orderFrontChannelsPanel:(id)sender
{
    [[[_server metaData] objectForKey:JKChannelManagerItem] orderFront];
}

- (void)orderFrontInspector:(id)sender
{
    [[JKInspector sharedInspector] orderFront];
}

- (void)orderFrontNickChangePanel:(id)sender
{
    [nickChangePanel makeKeyAndOrderFront:nil];
    while ([NSApp runModalForWindow:nickChangePanel] == NSRunContinuesResponse);
}

- (void)nickChangeOkClicked:(id)sender
{
    [[sender window] orderOut:nil];

    [_server sendEvent:[IRCEvent nicknameChangeEventWithNickname:[nickTextField stringValue]]];
    [[_server localUser] setNickname:[nickTextField stringValue]];
    
    [NSApp stopModal];
}

- (NSArray *)servers
{
    return [NSArray arrayWithObject:_server];
}

- (NSString *)applicationVersion
{
    return @"0.14";
}

- (NSArray *)apprentices
{
    return _apprentices;
}

- (NSDictionary *)channelDocuments
{
    return _channelDocuments;
}

- (NSDictionary *)queryDocuments
{
    return _queryDocuments;
}

- (NSMenu *)apprenticesMenu
{
    return apprenticesMenu;
}

- (void)addChannelDocument:(JKChannelDocument *)channelDocument
{
    [_channelDocuments setObject:channelDocument forKey:[[channelDocument channel] name]];
}

- (void)addQueryDocument:(JKQueryDocument *)queryDocument
{
    [_queryDocuments setObject:queryDocument forKey:[[queryDocument user] nickname]];
}

- (void)serverDidConnect:(NSNotification *)notification
{
    IRCServer *server = [notification object];

    [[server eventProcessor] addResponder:self];
    [[server metaData] setObject:[[[JKChannelManager alloc] initWithServer:server] autorelease]
                          forKey:JKChannelManagerItem];
}

- (void)serverWillTerminate:(NSNotification *)notification
{
    // we store the manager reference in the server's metadata, so we can deallocate
    // and close its panel (if applicable) on disconnect.
    JKChannelManager *manager = [[_server metaData] objectForKey:JKChannelManagerItem];
    [[_server metaData] removeObjectForKey:JKChannelManagerItem];
    [manager close];

    [[_server eventProcessor] removeResponder:self];

    [_server release];
    _server = nil;
}

- (void)channelDocumentWillClose:(NSNotification *)notification
{
    JKChannelDocument *document = [notification object];
    [_channelDocuments removeObjectForKey:[[document channel] name]];
}

- (void)serverDidERROR:(IRCServer *)server event:(IRCEvent *)event
{
    TCShowModelessAlertPanel(TCLocalizedStringForKey(@"serverError"),
                             TCLocalizedStringForKey(@"serverErrorFormat"),
                             [_server hostName], [[event arguments] objectAtIndex:0]);
}

- (void)serverDidJOIN:(IRCServer *)server event:(IRCEvent *)event
{
    if ([[[event prefix] nicknameComponent] isEqualToStringIgnoringCase:[[_server localUser] nickname]]) {
        IRCChannel *channel = [[_server channelDictionary] objectForKey:[[event arguments] objectAtIndex:0]];
        [self addChannelDocument:[JKChannelDocument channelDocumentWithChannel:channel]];
    }
}

- (void)serverDidPRIVMSG:(IRCServer *)server event:(IRCEvent *)event
{
    NSString *target = [[event arguments] objectAtIndex:0];
    NSString *sender = [[event prefix] nicknameComponent];

    if (![target hasPrefix:@"#"]) {
        if (![[[event arguments] objectAtIndex:1] hasPrefix:IRCCTCPDelimiterCharacter]) {
            if ([_queryDocuments objectForKey:sender] == nil) {
                IRCUser *user = [IRCUser userWithNickname:sender server:server];

                [self addQueryDocument:[JKQueryDocument queryDocumentWithUser:user]];
                [[_server eventProcessor] dispatchEvent:event
                                              responder:[_queryDocuments objectForKey:sender]];
            }

            [[[_queryDocuments objectForKey:sender] window] orderFront:nil];
        }
    }
}

// illegal nickname. present dialog box. set the info field to the reason
// why change is necessary; suggest alternate nickname.
- (void)serverDid433:(IRCServer *)server event:(IRCEvent *)event
{
    NSString *oldInfoMsg;

    if ([[[event arguments] objectAtIndex:1] isEqualToStringIgnoringCase:[[_server localUser] nickname]]) {
        oldInfoMsg = [nickInfoField stringValue];
        [nickInfoField setStringValue:[[event arguments] objectAtIndex:2]];
        [nickTextField setStringValue:[[[_server localUser] nickname] stringByAppendingString:@"_"]];

        // loop until "set" button hit
        while ([NSApp runModalForWindow:nickChangePanel] == NSRunContinuesResponse);
        [nickInfoField setStringValue:oldInfoMsg];
    }
}

@end
