#import "JediKnight.h"

@implementation JKOutputManager

+ (void)initialize
{
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            @"<%@>", @"JKOutputManagerDefaultMessageOutputFormat",
            @"*", @"JKOutputManagerDefaultActedActionOutputPrefix",
            @">", @"JKOutputManagerDefaultSpokenMessageOutputPrefix",
            @"***", @"JKOutputManagerDefaultErrorOutputPrefix",
            @"* %@", @"JKOutputManagerDefaultActionOutputFormat",
            @"-%@-", @"JKOutputManagerDefaultNoticeOutputFormat",
            nil]];
}

+ (NSString *)defaultMessageOutputFormat
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"JKOutputManagerDefaultMessageOutputFormat"];
}

+ (NSString *)defaultActedActionOutputPrefix
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"JKOutputManagerDefaultActedActionOutputPrefix"];
}

+ (NSString *)defaultSpokenMessageOutputPrefix
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"JKOutputManagerDefaultSpokenMessageOutputPrefix"];
}

+ (NSString *)defaultErrorOutputPrefix
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"JKOutputManagerDefaultErrorOutputPrefix"];
}

+ (NSString *)defaultActionOutputFormat
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"JKOutputManagerDefaultActionOutputFormat"];
}

+ (NSString *)defaultNoticeOutputFormat
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"JKOutputManagerDefaultNoticeOutputFormat"];
}

+ (JKOutputManager *)outputManagerWithTextView:(NSTextView *)textView
{
    return [[[self alloc] initWithTextView:textView] autorelease];
}

- (id)initWithTextView:(NSTextView *)textView
{
    if (self = [super init]) {
        _textView = [textView retain];

        _messageOutputFormat = [[self class] defaultMessageOutputFormat];
        _spokenMessageOutputPrefix = [[self class] defaultSpokenMessageOutputPrefix];
        _errorOutputPrefix = [[self class] defaultErrorOutputPrefix];
        _actionOutputFormat = [[self class] defaultActionOutputFormat];
        _actedActionOutputPrefix = [[self class] defaultActedActionOutputPrefix];
        _noticeOutputFormat = [[self class] defaultNoticeOutputFormat];

        _prependNewline = NO;
    }

    return self;
}

- (void)dealloc
{
    [_textView release];
    [_messageOutputFormat release];
    [_spokenMessageOutputPrefix release];
    [_errorOutputPrefix release];
    [_actionOutputFormat release];
    [_actedActionOutputPrefix release];
    [_noticeOutputFormat release];

    [super dealloc];
}

- (void)setPrependNewline:(BOOL)flag
{
    _prependNewline = flag;
}

- (void)appendMessage:(NSString *)message speaker:(NSString *)speaker
{
    NSString *prefix = [NSString stringWithFormat:_messageOutputFormat, speaker];

    [_textView appendWithFormat:@"%@%@ %@", (_prependNewline ? @"\n" : @""), prefix, message];
    [_textView scrollRangeToVisible:NSMakeRange([[_textView string] length], 0)];

    if (!_prependNewline)
        _prependNewline = YES;
}

- (void)appendSpokenMessage:(NSString *)spokenMessage
{
    [_textView appendWithFormat:@"%@%@ %@", (_prependNewline ? @"\n" : @""), _spokenMessageOutputPrefix, spokenMessage];
    [_textView scrollRangeToVisible:NSMakeRange([[_textView string] length], 0)];

    if (!_prependNewline)
        _prependNewline = YES;
}

- (void)appendClientMessage:(NSString *)error, ...
{
    va_list argList;
    NSString *string;

    va_start(argList, error);
    string = [[[NSString alloc] initWithFormat:error arguments:argList] autorelease];
    va_end(argList);

    [_textView appendWithFormat:@"%@%@ %@", (_prependNewline ? @"\n" : @""), _errorOutputPrefix, string];
    [_textView scrollRangeToVisible:NSMakeRange([[_textView string] length], 0)];

    if (!_prependNewline)
        _prependNewline = YES;
}

- (void)appendAction:(NSString *)action actor:(NSString *)actor
{
    NSString *prefix = [NSString stringWithFormat:_actionOutputFormat, actor];

    [_textView appendWithFormat:@"%@%@ %@", (_prependNewline ? @"\n" : @""), prefix, action];
    [_textView scrollRangeToVisible:NSMakeRange([[_textView string] length], 0)];

    if (!_prependNewline)
        _prependNewline = YES;
}

- (void)appendActedAction:(NSString *)action actor:(NSString *)actor
{
    [_textView appendWithFormat:@"%@%@ %@ %@",
        (_prependNewline ? @"\n" : @""),
        _actedActionOutputPrefix,
        actor,
        action];
    [_textView scrollRangeToVisible:NSMakeRange([[_textView string] length], 0)];

    if (!_prependNewline)
        _prependNewline = YES;
}

- (void)appendNotice:(NSString *)notice speaker:(NSString *)speaker
{
    NSString *prefix = [NSString stringWithFormat:_noticeOutputFormat, speaker];

    [_textView appendWithFormat:@"%@%@ %@", (_prependNewline ? @"\n" : @""), prefix, notice];
    [_textView scrollRangeToVisible:NSMakeRange([[_textView string] length], 0)];

    if (!_prependNewline)
        _prependNewline = YES;
}

@end
