#import "JediKnight.h"

@implementation JKPersistentServerRepresentation

+ (JKPersistentServerRepresentation *)serverRepresentationWithServer:(IRCServer *)server
{
    return [[[self alloc] initWithServer:server] autorelease];
}

- (id)initWithServer:(IRCServer *)server
{
    if (self = [super init]) {
        _hostName = [[server hostName] retain];
        _port = [server port];
        _nickname = [[[server localUser] nickname] retain];
        _userDescription = [[[server localUser] userDescription] retain];
        _knownChannels = [[[server channelDictionary] allKeys] retain];
    }

    return self;
}

- (void)dealloc
{
    [_hostName release];
    [_nickname release];
    [_userDescription release];
    [_knownChannels release];
    
    [super dealloc];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@[0x%lx] hostName: %@ port: %d nickname: %@ userDescription: %@ knownChannels: %@>",
        [[self class] description], self,
        _hostName, _port, _nickname, _userDescription, [_knownChannels description]];
}

- (NSString *)hostName
{
    return _hostName;
}

- (int)port
{
    return _port;
}

- (NSString *)nickname
{
    return _nickname;
}

- (NSString *)userDescription
{
    return _userDescription;
}

- (NSArray *)knownChannels
{
    return _knownChannels;
}

- (id)initWithCoder:(NSCoder *)coder
{
    if (self = [super init]) {
        _nickname = [[coder decodeObject] retain];
        [coder decodeValueOfObjCType:@encode(int) at:&_port];
        _hostName = [[coder decodeObject] retain];
        _userDescription = [[coder decodeObject] retain];
        _knownChannels = [[coder decodeObject] retain];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeObject:_nickname];
    [coder encodeValueOfObjCType:@encode(int) at:&_port];
    [coder encodeObject:_hostName];
    [coder encodeObject:_userDescription];
    [coder encodeObject:_knownChannels];
}

@end
